module Calculator.Primitives where

import Text.ParserCombinators.Parsec.Error (ParseError)

type Number = Double

data Expr = Function String Expr
          | Constant Number
          | UnOp Operator Expr
          | BinOp (Expr, [(Operator, Expr)])
          | InvalidError String

data Operator = UnaryOp  (Number -> Number)
              | BinaryOp (Number -> Number -> Number)

evalFaliure :: String
evalFaliure = "Evaluation Failed"

fromConst :: Expr -> Number
fromConst (Constant c) = c
fromConst _ = error "Error: Making number from expr"

isConst :: Expr -> Bool
isConst (Constant _) = True
isConst _ = False

result :: Either ParseError Expr -> String
result (Right (Constant c)) = " == " ++ show c
result (Right (InvalidError e)) = " !! " ++ e
result (Left e) = show e
result _ = evalFaliure

binaryOps :: [(Char, Operator)]
binaryOps = [ ('+', BinaryOp (+))
            , ('-', BinaryOp (-))
            , ('*', BinaryOp (*))
            , ('/', BinaryOp (/))
            , ('^', BinaryOp (**))
            ]

dispatch :: (RealFrac a, Floating a) => [(String, a -> a)]
dispatch = [ ("sin", sin)
           , ("cos", cos)
           , ("tan", tan)
           , ("asin", asin)
           , ("acos", acos)
           , ("atan", atan)
           , ("sinh", sinh)
           , ("cosh", cosh)
           , ("tanh", tanh)
           , ("exp", exp)
           , ("log", log)
           , ("log10", logBase 10)
           , ("sqrt", sqrt)
           , ("ceil", fromInteger . ceiling)
           , ("floor", fromInteger . floor)
           , ("abs", abs)
           ]
